/*

    CHARS.CH

    Output character definitions.  This file includes only non-printing
    characters which have an effect on output, and extended ASCII
    characters which are not on the standard American keyboard.
    ASC() and CHR() handle conversions of keyboard-accessible
    characters.  KEYS.CH handles conversions of non-printing input
    characters.

    Warning:  Only the first ten characters of any identifier name in
    Clipper are significant, including #DEFINE symbols.  Many of the names
    in this file are longer, but this is for mnemonic purposes only.

    Kevin Carmody, September 1997.

*/

//  Clipper text control codes

/*
    Soft carriage returns are used in memo fields.  A hard or soft
    carriage return must be preceded by a space and followed by a line
    feed to have effect.  MEMOLINE() puts a space at the end of a line
    that would be broken by a soft carriage return in the corresponding
    MEMOEDIT().
*/

#DEFINE CRLF                CHR(13) + CHR(10)
#DEFINE C_HARD_NEW_LINE     CHR(13) + CHR(10)
#DEFINE C_SOFT_NEW_LINE     CHR(141) + CHR(10)
#DEFINE C_CR                CHR(13)
#DEFINE C_HARD_CR           CHR(13)
#DEFINE C_SOFT_CR           CHR(141)
#DEFINE C_TAB               CHR(9)
#DEFINE C_LINE_FEED         CHR(10)
#DEFINE C_FORM_FEED         CHR(12)

//  General-purpose symbols

#DEFINE C_SMALL_DOT         CHR(250)
#DEFINE C_MEDIUM_DOT        CHR(249)
#DEFINE C_BIG_DOT           CHR(7)

#DEFINE C_HOLE              CHR(8)
#DEFINE C_RING              CHR(9)
#DEFINE C_RING_HOLE         CHR(10)

#DEFINE C_LEFT_TRIANGLE     CHR(17)
#DEFINE C_RIGHT_TRIANGLE    CHR(16)
#DEFINE C_UP_TRIANGLE       CHR(30)
#DEFINE C_DOWN_TRIANGLE     CHR(31)

#DEFINE C_LEFT_ARROW        CHR(27)
#DEFINE C_RIGHT_ARROW       CHR(26)
#DEFINE C_UP_ARROW          CHR(24)
#DEFINE C_DOWN_ARROW        CHR(25)

#DEFINE C_LEFT_RIGHT_ARROW  CHR(29)
#DEFINE C_UPDOWN_ARROW      CHR(18)
#DEFINE C_UPDOWN_BASE       CHR(23)

//  Graphics characters

/*  Usage: TL = top left
           TR = top right
           BL = bottom left

    There is no bottom right corner in this series.
    See also the box drawing characters.
*/

#DEFINE C_TL_CORNER         CHR(169)
#DEFINE C_TR_CORNER         CHR(170)
#DEFINE C_BL_CORNER         CHR(28)

#DEFINE C_LOW_BLOCK         CHR(22)
#DEFINE C_CENTER_BLOCK      CHR(254)

#DEFINE C_BLOCK_LEFT        CHR(221)
#DEFINE C_BLOCK_RIGHT       CHR(222)
#DEFINE C_BLOCK_TOP         CHR(223)
#DEFINE C_BLOCK_BOTTOM      CHR(220)

#DEFINE C_LIGHT_FILL        CHR(176)
#DEFINE C_MEDIUM_FILL       CHR(177)
#DEFINE C_HEAVY_FILL        CHR(178)
#DEFINE C_BLOCK_FILL        CHR(219)

//  Box drawing characters

/*
    Usage: These characters can draw single or double boxes or join two
    boxes of either type.  In the character name, the one or two letters
    after the 'C' indicate whether single or double boxes are involved:
    'S' stands for a single box, and 'D' stands for double.

    If there is one letter after the 'C', then the character is drawn only
    on the side of a box.  These characters may be either for a vertical
    line or a horizontal line and are named 'VERT' or 'HORIZ'.

    If there are two letters after the 'C', then the character is involved
    at an intersection of two box lines, one vertical and one horizontal.
    The first letter after the 'C' indicates the type of the vertical line
    (single or double), and the second letter indicates the type of the
    horizontal line.  For example, a name starting with 'C_SD' is involved
    at the intersection of a single vertical line and a double horizontal
    line.

    There are three types of intersection characters, depending on how many
    'legs' the character has.  A leg is a half-line coming out from the
    center.  The three types of intersection characters are: an 'L' (or
    corner), with two legs; a 'T', with three legs; and a cross, with four
    legs.

    There are four kinds of L's: top left (TL), top right (TR), bottom left
    (BL), and bottom right (BR), these being the box positions they should
    be used in; these are also the two sides of the character which do not
    have legs.

    There are four types of T's:  left, right, top, and bottom, these being
    the side of the box they should be used on; this is also the side of
    the character which does not have a leg.

    There is only one type of cross.
*/

#DEFINE C_S_VERT            CHR(179)
#DEFINE C_S_HORIZ           CHR(196)

#DEFINE C_SS_L_TL           CHR(218)
#DEFINE C_SS_L_TR           CHR(191)
#DEFINE C_SS_L_BL           CHR(192)
#DEFINE C_SS_L_BR           CHR(217)
#DEFINE C_SS_T_LEFT         CHR(195)
#DEFINE C_SS_T_RIGHT        CHR(180)
#DEFINE C_SS_T_TOP          CHR(194)
#DEFINE C_SS_T_BOTTOM       CHR(193)
#DEFINE C_SS_CROSS          CHR(197)

#DEFINE C_SD_L_TL           CHR(213)
#DEFINE C_SD_L_TR           CHR(184)
#DEFINE C_SD_L_BL           CHR(212)
#DEFINE C_SD_L_BR           CHR(190)
#DEFINE C_SD_T_LEFT         CHR(198)
#DEFINE C_SD_T_RIGHT        CHR(181)
#DEFINE C_SD_T_TOP          CHR(209)
#DEFINE C_SD_T_BOTTOM       CHR(207)
#DEFINE C_SD_CROSS          CHR(216)

#DEFINE C_D_VERT            CHR(186)
#DEFINE C_D_HORIZ           CHR(205)

#DEFINE C_DD_L_TL           CHR(201)
#DEFINE C_DD_L_TR           CHR(187)
#DEFINE C_DD_L_BR           CHR(188)
#DEFINE C_DD_L_BL           CHR(200)
#DEFINE C_DD_T_LEFT         CHR(204)
#DEFINE C_DD_T_RIGHT        CHR(185)
#DEFINE C_DD_T_TOP          CHR(203)
#DEFINE C_DD_T_BOTTOM       CHR(202)
#DEFINE C_DD_CROSS          CHR(206)

#DEFINE C_DS_L_TL           CHR(214)
#DEFINE C_DS_L_TR           CHR(183)
#DEFINE C_DS_L_BL           CHR(211)
#DEFINE C_DS_L_BR           CHR(189)
#DEFINE C_DS_T_LEFT         CHR(199)
#DEFINE C_DS_T_RIGHT        CHR(182)
#DEFINE C_DS_T_TOP          CHR(210)
#DEFINE C_DS_T_BOTTOM       CHR(208)
#DEFINE C_DS_CROSS          CHR(215)

//  Uppercase international letters

#DEFINE C_CAP_A_UMLAUT      CHR(142)
#DEFINE C_CAP_AE            CHR(146)
#DEFINE C_CAP_A_CIRCLE      CHR(143)
#DEFINE C_CAP_C_CEDILLA     CHR(128)
#DEFINE C_CAP_E_AIGU        CHR(144)
#DEFINE C_CAP_N_TILDE       CHR(165)
#DEFINE C_CAP_O_UMLAUT      CHR(153)
#DEFINE C_CAP_U_UMLAUT      CHR(154)

//  Lowercase international letters

#DEFINE C_A_AIGU            CHR(160)
#DEFINE C_A_CIRCUMFLEX      CHR(131)
#DEFINE C_A_CIRCLE          CHR(134)
#DEFINE C_A_GRAVE           CHR(133)
#DEFINE C_A_UMLAUT          CHR(132)
#DEFINE C_AE                CHR(145)
#DEFINE C_C_CEDILLA         CHR(135)
#DEFINE C_E_AIGU            CHR(130)
#DEFINE C_E_CIRCUMFLEX      CHR(136)
#DEFINE C_E_GRAVE           CHR(138)
#DEFINE C_E_UMLAUT          CHR(137)
#DEFINE C_I_AIGU            CHR(161)
#DEFINE C_I_CIRCUMFLEX      CHR(140)
#DEFINE C_I_GRAVE           CHR(141)
#DEFINE C_I_UMLAUT          CHR(139)
#DEFINE C_N_TILDE           CHR(164)
#DEFINE C_O_AIGU            CHR(162)
#DEFINE C_O_CIRCUMFLEX      CHR(147)
#DEFINE C_O_GRAVE           CHR(149)
#DEFINE C_O_UMLAUT          CHR(148)
#DEFINE C_U_AIGU            CHR(163)
#DEFINE C_U_CIRCUMFLEX      CHR(150)
#DEFINE C_U_GRAVE           CHR(151)
#DEFINE C_U_UMLAUT          CHR(129)
#DEFINE C_Y_UMLAUT          CHR(152)

//  Punctuation

#DEFINE C_HOOK_INVERSE      CHR(168)
#DEFINE C_BANG_INVERSE      CHR(173)
#DEFINE C_LEFT_GUILLEMET    CHR(174)
#DEFINE C_RIGHT_GUILLEMET   CHR(175)
#DEFINE C_TWO_BANG          CHR(19)
#DEFINE C_PARAGRAPH         CHR(20)
#DEFINE C_SECTION           CHR(21)

#DEFINE C_HALF              CHR(171)
#DEFINE C_QUARTER           CHR(172)
#DEFINE C_CENT              CHR(155)
#DEFINE C_POUND             CHR(156)
#DEFINE C_YEN               CHR(157)
#DEFINE C_PT                CHR(158)
#DEFINE C_A_RAISED          CHR(166)
#DEFINE C_O_RAISED          CHR(167)

//  Lowercase Greek letters

#DEFINE C_ALPHA             CHR(224)
#DEFINE C_BETA              CHR(225)
#DEFINE C_DELTA             CHR(235)
#DEFINE C_EPSILON           CHR(238)
#DEFINE C_THETA             CHR(233)
#DEFINE C_MU                CHR(230)
#DEFINE C_PI                CHR(227)
#DEFINE C_SIGMA             CHR(229)
#DEFINE C_TAU               CHR(231)
#DEFINE C_PHI               CHR(237)

//  Uppercase Greek letters

#DEFINE C_CAP_GAMMA         CHR(226)
#DEFINE C_CAP_PHI           CHR(232)
#DEFINE C_CAP_SIGMA         CHR(228)
#DEFINE C_CAP_OMEGA         CHR(234)

//  Mathematical symbols

#DEFINE C_INFINITY          CHR(236)
#DEFINE C_FANCY_F           CHR(159)
#DEFINE C_DEGREE            CHR(248)
#DEFINE C_N_EXPONENT        CHR(252)
#DEFINE C_2_EXPONENT        CHR(253)

#DEFINE C_EQUIVALENCE       CHR(240)
#DEFINE C_CONGRUENCE        CHR(247)
#DEFINE C_GREATER_EQUAL     CHR(242)
#DEFINE C_LESS_EQUAL        CHR(243)

#DEFINE C_PLUS_MINUS        CHR(241)
#DEFINE C_DIVIDED           CHR(246)
#DEFINE C_SQUARE_ROOT       CHR(251)
#DEFINE C_TOP_INTEGRAL      CHR(244)
#DEFINE C_BOTTOM_INTEGRAL   CHR(245)
#DEFINE C_INTERSECTION      CHR(239)

//  Miscellaneous symbols

#DEFINE C_HAPPY_LIGHT       CHR(1)
#DEFINE C_HAPPY_DARK        CHR(2)

#DEFINE C_HEART             CHR(3)
#DEFINE C_DIAMOND           CHR(4)
#DEFINE C_CLUB              CHR(5)
#DEFINE C_SPADE             CHR(6)

#DEFINE C_MALE              CHR(11)
#DEFINE C_FEMALE            CHR(12)
#DEFINE C_NOTE              CHR(13)
#DEFINE C_TWO_NOTE          CHR(14)
#DEFINE C_SPARKLE           CHR(15)
#DEFINE C_HOUSE             CHR(127)

/*

// Definitions in numeric order

#DEFINE C_HAPPY_LIGHT       CHR(  1)
#DEFINE C_HAPPY_DARK        CHR(  2)
#DEFINE C_HEART             CHR(  3)
#DEFINE C_DIAMOND           CHR(  4)
#DEFINE C_CLUB              CHR(  5)
#DEFINE C_SPADE             CHR(  6)
#DEFINE C_BIG_DOT           CHR(  7)
#DEFINE C_HOLE              CHR(  8)
#DEFINE C_TAB               CHR(  9)
#DEFINE C_RING              CHR(  9)
#DEFINE C_RING_HOLE         CHR( 10)
#DEFINE C_LINE_FEED         CHR( 10)
#DEFINE C_MALE              CHR( 11)
#DEFINE C_FEMALE            CHR( 12)
#DEFINE C_FORM_FEED         CHR( 12)
#DEFINE C_NOTE              CHR( 13)
#DEFINE C_CR                CHR( 13)
#DEFINE C_HARD_CR           CHR( 13)
#DEFINE C_HARD_NEW_LINE     CHR( 13) + CHR( 10)
#DEFINE C_NL                CHR( 13) + CHR( 10)
#DEFINE C_TWO_NOTE          CHR( 14)
#DEFINE C_SPARKLE           CHR( 15)
#DEFINE C_RIGHT_TRIANGLE    CHR( 16)
#DEFINE C_LEFT_TRIANGLE     CHR( 17)
#DEFINE C_UPDOWN_ARROW      CHR( 18)
#DEFINE C_TWO_BANG          CHR( 19)
#DEFINE C_PARAGRAPH         CHR( 20)
#DEFINE C_SECTION           CHR( 21)
#DEFINE C_LOW_BLOCK         CHR( 22)
#DEFINE C_UPDOWN_BASE       CHR( 23)
#DEFINE C_UP_ARROW          CHR( 24)
#DEFINE C_DOWN_ARROW        CHR( 25)
#DEFINE C_RIGHT_ARROW       CHR( 26)
#DEFINE C_LEFT_ARROW        CHR( 27)
#DEFINE C_BL_CORNER         CHR( 28)
#DEFINE C_LEFT_RIGHT_ARROW  CHR( 29)
#DEFINE C_UP_TRIANGLE       CHR( 30)
#DEFINE C_DOWN_TRIANGLE     CHR( 31)
#DEFINE C_HOUSE             CHR(127)
#DEFINE C_CAP_C_CEDILLA     CHR(128)
#DEFINE C_U_UMLAUT          CHR(129)
#DEFINE C_E_AIGU            CHR(130)
#DEFINE C_A_CIRCUMFLEX      CHR(131)
#DEFINE C_A_UMLAUT          CHR(132)
#DEFINE C_A_GRAVE           CHR(133)
#DEFINE C_A_CIRCLE          CHR(134)
#DEFINE C_C_CEDILLA         CHR(135)
#DEFINE C_E_CIRCUMFLEX      CHR(136)
#DEFINE C_E_UMLAUT          CHR(137)
#DEFINE C_E_GRAVE           CHR(138)
#DEFINE C_I_UMLAUT          CHR(139)
#DEFINE C_I_CIRCUMFLEX      CHR(140)
#DEFINE C_I_GRAVE           CHR(141)
#DEFINE C_SOFT_CR           CHR(141)
#DEFINE C_SOFT_NEW_LINE     CHR(141) + CHR( 10)
#DEFINE C_CAP_A_UMLAUT      CHR(142)
#DEFINE C_CAP_A_CIRCLE      CHR(143)
#DEFINE C_CAP_E_AIGU        CHR(144)
#DEFINE C_AE                CHR(145)
#DEFINE C_CAP_AE            CHR(146)
#DEFINE C_O_CIRCUMFLEX      CHR(147)
#DEFINE C_O_UMLAUT          CHR(148)
#DEFINE C_O_GRAVE           CHR(149)
#DEFINE C_U_CIRCUMFLEX      CHR(150)
#DEFINE C_U_GRAVE           CHR(151)
#DEFINE C_Y_UMLAUT          CHR(152)
#DEFINE C_CAP_O_UMLAUT      CHR(153)
#DEFINE C_CAP_U_UMLAUT      CHR(154)
#DEFINE C_CENT              CHR(155)
#DEFINE C_POUND             CHR(156)
#DEFINE C_YEN               CHR(157)
#DEFINE C_PT                CHR(158)
#DEFINE C_FANCY_F           CHR(159)
#DEFINE C_A_AIGU            CHR(160)
#DEFINE C_I_AIGU            CHR(161)
#DEFINE C_O_AIGU            CHR(162)
#DEFINE C_U_AIGU            CHR(163)
#DEFINE C_N_TILDE           CHR(164)
#DEFINE C_CAP_N_TILDE       CHR(165)
#DEFINE C_A_RAISED          CHR(166)
#DEFINE C_O_RAISED          CHR(167)
#DEFINE C_HOOK_INVERSE      CHR(168)
#DEFINE C_TL_CORNER         CHR(169)
#DEFINE C_TR_CORNER         CHR(170)
#DEFINE C_HALF              CHR(171)
#DEFINE C_QUARTER           CHR(172)
#DEFINE C_BANG_INVERSE      CHR(173)
#DEFINE C_LEFT_GUILLEMET    CHR(174)
#DEFINE C_RIGHT_GUILLEMET   CHR(175)
#DEFINE C_LIGHT_FILL        CHR(176)
#DEFINE C_MEDIUM_FILL       CHR(177)
#DEFINE C_HEAVY_FILL        CHR(178)
#DEFINE C_S_VERT            CHR(179)
#DEFINE C_SS_T_RIGHT        CHR(180)
#DEFINE C_SD_T_RIGHT        CHR(181)
#DEFINE C_DS_T_RIGHT        CHR(182)
#DEFINE C_DS_L_TR           CHR(183)
#DEFINE C_SD_L_TR           CHR(184)
#DEFINE C_DD_T_RIGHT        CHR(185)
#DEFINE C_D_VERT            CHR(186)
#DEFINE C_DD_L_TR           CHR(187)
#DEFINE C_DD_L_BR           CHR(188)
#DEFINE C_DS_L_BR           CHR(189)
#DEFINE C_SD_L_BR           CHR(190)
#DEFINE C_SS_L_TR           CHR(191)
#DEFINE C_SS_L_BL           CHR(192)
#DEFINE C_SS_T_BOTTOM       CHR(193)
#DEFINE C_SS_T_TOP          CHR(194)
#DEFINE C_SS_T_LEFT         CHR(195)
#DEFINE C_S_HORIZ           CHR(196)
#DEFINE C_SS_CROSS          CHR(197)
#DEFINE C_SD_T_LEFT         CHR(198)
#DEFINE C_DS_T_LEFT         CHR(199)
#DEFINE C_DD_L_BL           CHR(200)
#DEFINE C_DD_L_TL           CHR(201)
#DEFINE C_DD_T_BOTTOM       CHR(202)
#DEFINE C_DD_T_TOP          CHR(203)
#DEFINE C_DD_T_LEFT         CHR(204)
#DEFINE C_D_HORIZ           CHR(205)
#DEFINE C_DD_CROSS          CHR(206)
#DEFINE C_SD_T_BOTTOM       CHR(207)
#DEFINE C_DS_T_BOTTOM       CHR(208)
#DEFINE C_SD_T_TOP          CHR(209)
#DEFINE C_DS_T_TOP          CHR(210)
#DEFINE C_DS_L_BL           CHR(211)
#DEFINE C_SD_L_BL           CHR(212)
#DEFINE C_SD_L_TL           CHR(213)
#DEFINE C_DS_L_TL           CHR(214)
#DEFINE C_DS_CROSS          CHR(215)
#DEFINE C_SD_CROSS          CHR(216)
#DEFINE C_SS_L_BR           CHR(217)
#DEFINE C_SS_L_TL           CHR(218)
#DEFINE C_BLOCK_FILL        CHR(219)
#DEFINE C_BLOCK_BOTTOM      CHR(220)
#DEFINE C_BLOCK_LEFT        CHR(221)
#DEFINE C_BLOCK_RIGHT       CHR(222)
#DEFINE C_BLOCK_TOP         CHR(223)
#DEFINE C_ALPHA             CHR(224)
#DEFINE C_BETA              CHR(225)
#DEFINE C_CAP_GAMMA         CHR(226)
#DEFINE C_PI                CHR(227)
#DEFINE C_CAP_SIGMA         CHR(228)
#DEFINE C_SIGMA             CHR(229)
#DEFINE C_MU                CHR(230)
#DEFINE C_TAU               CHR(231)
#DEFINE C_CAP_PHI           CHR(232)
#DEFINE C_THETA             CHR(233)
#DEFINE C_CAP_OMEGA         CHR(234)
#DEFINE C_DELTA             CHR(235)
#DEFINE C_INFINITY          CHR(236)
#DEFINE C_PHI               CHR(237)
#DEFINE C_EPSILON           CHR(238)
#DEFINE C_INTERSECTION      CHR(239)
#DEFINE C_EQUIVALENCE       CHR(240)
#DEFINE C_PLUS_MINUS        CHR(241)
#DEFINE C_GREATER_EQUAL     CHR(242)
#DEFINE C_LESS_EQUAL        CHR(243)
#DEFINE C_TOP_INTEGRAL      CHR(244)
#DEFINE C_BOTTOM_INTEGRAL   CHR(245)
#DEFINE C_DIVIDED           CHR(246)
#DEFINE C_CONGRUENCE        CHR(247)
#DEFINE C_DEGREE            CHR(248)
#DEFINE C_MEDIUM_DOT        CHR(249)
#DEFINE C_SMALL_DOT         CHR(250)
#DEFINE C_SQUARE_ROOT       CHR(251)
#DEFINE C_N_EXPONENT        CHR(252)
#DEFINE C_2_EXPONENT        CHR(253)
#DEFINE C_CENTER_BLOCK      CHR(254)

*/