*****************************************************************************
*
* ERRORSYS
*
* Revised Clipper error handler.  Same as standard Clipper error handler
* except that alert box translates DOS error code into text message.
*
*****************************************************************************

#INCLUDE "ERROR.CH"
// Put messages to STDERR.
#COMMAND ? <list,...>   =>  ?? CHR(13) + CHR(10) ; ?? <list>
#COMMAND ?? <list,...>  =>  OUTERR(<list>)
// Used below.
#DEFINE NTrim(n)    LTRIM(STR(n))

*****************************************************************************
*  ErrorSys()
*  Automatically executes at startup.
*****************************************************************************

PROCEDURE ErrorSys()
  ERRORBLOCK( {|oErr| DefError(oErr)} )
RETURN  //  ErrorSys

*****************************************************************************
* DefError()
* Display alert box and process options.
*****************************************************************************

STATIC FUNCTION DefError(oErr)

LOCAL nItem    := 1
LOCAL aMessage := {}
LOCAL cMessage := ''
LOCAL aOptions := {"Break", "Quit"}
LOCAL nChoice  := 0

// By default, division by zero yields zero.
IF oErr:GENCODE == EG_ZERODIV
  RETURN 0
END

// For network open error, set NETERR() and subsystem default.
IF oErr:GENCODE == EG_OPEN .AND. oErr:OSCODE == 32 .AND. oErr:CANDEFAULT
  NETERR(.T.)
  RETURN .F.                  // NOTE
END

// For lock error during APPEND BLANK, set NETERR() and subsystem default.
IF oErr:GENCODE == EG_APPENDLOCK .AND. oErr:CANDEFAULT
  NETERR(.T.)
  RETURN .F.                  // NOTE
END

// Build error message.
aMessage := ErrorMessage(oErr)
cMessage := aMessage[1] + IF( EMPTY(aMessage[2]), "", ";" + aMessage[2] )

// Build options array.
IF oErr:CANRETRY
  AADD(aOptions, "Retry")
END
IF oErr:CANDEFAULT
  AADD(aOptions, "Default")
END

// Put up alert box.
WHILE EMPTY(nChoice)
  nChoice := ALERT( cMessage, aOptions )
END

DO CASE
CASE aOptions[nChoice] == "Break"
  BREAK oErr
CASE aOptions[nChoice] == "Retry"
  RETURN .T.
CASE aOptions[nChoice] == "Default"
  RETURN .F.
OTHERWISE

  // Display message and traceback.
  ? aMessage[1]
  IF !EMPTY( aMessage[2] )
    ? aMessage[2]
  END
  WHILE !EMPTY(PROCNAME(++nItem))
    ? "Called from", TRIM(PROCNAME(nItem)) + ;
      "(" + NTrim(PROCLINE(nItem)) + ")"
  END
  // Give up.
  ERRORLEVEL(1)
  QUIT

END

RETURN .F.  //  DefError

*****************************************************************************
* ErrorMessage()
* Build error message text from error object. 
*****************************************************************************

STATIC FUNCTION ErrorMessage(oErr)

LOCAL cMessage := ""
LOCAL aMessage := {"", ""}

// Start error message.
cMessage := IF( oErr:SEVERITY > ES_WARNING, "Error ", "Warning " )

// Add subsystem name if available.
cMessage += IF( VALTYPE(oErr:SUBSYSTEM) == "C", oErr:SUBSYSTEM(), "???" )

// Add subsystem's error code if available.
cMessage += IF( VALTYPE(oErr:SUBCODE) == "N", ;
  "/" + NTrim(oErr:SUBCODE), "/???" )

// Add error description if available.
cMessage += IF( VALTYPE(oErr:DESCRIPTION) == "C", ;
  "  " + oErr:DESCRIPTION, "")

// Add filename.
cMessage += IF( !EMPTY(oErr:FILENAME), ": " + oErr:FILENAME, "")

// Add operation.
cMessage += IF( !EMPTY(oErr:OPERATION), ": " + oErr:OPERATION, "")

// Make above message the first message element.
aMessage[1] := cMessage

// Add DOS error code in second element.
IF !EMPTY(oErr:OSCODE)
  aMessage[2] := "DOS Error " + NTrim(oErr:OSCODE) + ": " + ;
    DosErrMessage(oErr:OSCODE)
END

RETURN aMessage  //  ErrorMessage

*****************************************************************************
* DosErrMessage
* Translate DOS error number to text.
*****************************************************************************

FUNCTION DosErrMessage(nDosErr)

LOCAL aError := {;
    { 1, 'Invalid function number'},;
    { 2, 'File not found'},;
    { 3, 'Path not found'},;
    { 4, 'Too many open files (no handles left)'},;
    { 5, 'Access denied'},;
    { 6, 'Invalid handle'},;
    { 7, 'Memory control blocks destroyed'},;
    { 8, 'Insufficient memory'},;
    { 9, 'Invalid memory block address'},;
    {10, 'Invalid environment'},;
    {11, 'Invalid format'},;
    {12, 'Invalid acccess code'},;
    {13, 'Invalid data'},;
    {15, 'Invalid drive'},;
    {16, 'Attempt to remove current directory'},;
    {17, 'Not same device'},;
    {18, 'No more files'},;
    {19, 'Attempt to write to write-protected device'},;
    {20, 'Unknown unit'},;
    {21, 'Drive not ready'},;
    {22, 'Unknown command'},;
    {23, 'Data error (CRC)'},;
    {24, 'Bad request structure length'},;
    {25, 'Seek error'},;
    {26, 'Unknown media type'},;
    {27, 'Sector not found'},;
    {28, 'Printer out of paper'},;
    {29, 'Write fault'},;
    {30, 'Read fault'},;
    {31, 'General failure'},;
    {32, 'Sharing violation'},;
    {33, 'Lock violation'},;
    {34, 'Invalid disk change'},;
    {35, 'FCB unavailable'},;
    {36, 'Sharing buffer overflow'},;
    {50, 'Network request not supported'},;
    {51, 'Remote computer not listening'},;
    {52, 'Duplicate name on network'},;
    {53, 'Network name not found'},;
    {54, 'Network busy'},;
    {55, 'Network device no longer exists'},;
    {56, 'Network BIOS command limit exceeded'},;
    {57, 'Network adapter hardware error'},;
    {58, 'Incorrect response from network'},;
    {59, 'Unexpected network error'},;
    {60, 'Incompatible remote adapter'},;
    {61, 'Print queue full'},;
    {62, 'Not enough space for print file'},;
    {63, 'Print file deleted (not enough space)'},;
    {64, 'Network name deleted'},;
    {65, 'Access denied'},;
    {66, 'Network device type incorrect'},;
    {67, 'Network name not found'},;
    {68, 'Network name limit exceeded'},;
    {69, 'Network BIOS session limit exceeded'},;
    {70, 'Temporarily paused'},;
    {71, 'Network request not accepted'},;
    {72, 'Print or disk redirection paused'},;
    {80, 'File already exists'},;
    {82, 'Cannot make directory entry'},;
    {83, 'Fail on INT 24H'},;
    {84, 'Too many redirections'},;
    {85, 'Duplicte redirection'},;
    {86, 'Invalid password'},;
    {87, 'Invalid parameter'},;
    {88, 'Network device fault'}}
LOCAL nEntry := ASCAN(aError, {|aRow| aRow[1] == nDosErr})
LOCAL cText  := IF(EMPTY(nEntry), 'Unknown', aError[nEntry][2])

RETURN cText  //  DosErrMessage

*****************************************************************************

// End of file ERRORSYS.PRG
